/****************************************************************************\
*   BBIO - this header file is part of Binary vIEW project (BIEW)            *
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
*   Copyrights:           1995, 2000 Nick Kurshev                            *
*   License:              See below                                          *
*   Author and developer: Nick Kurshev                                       *
*   Requirement:          ANSI C compatible development system               *
*   Original file name:   lib/bbio.h                                         *
*---------------------- [ D e s c r i p t i o n ] ---------------------------*
*  This file contain prototypes of BBIO technology functions.                *
*-------------------------- [ L i c e n c e ] -------------------------------*
* The Binary vIEW (BIEW) is copyright (C) 1995 Nick Kurshev.                 *
* All rights reserved. This software is redistributable under the licence    *
* given in the file "Licence.en" ("Licence.ru" in russian translation)       *
* distributed in the BIEW archive.                                           *
\****************************************************************************/
#ifndef __BBIO_H
#define __BBIO_H

#ifndef __BIEWLIB_H
#include "lib/biewlib.h"
#endif
/******************************************************************\
*  Buffered binary file streams input/output section               *
*  Helpful for read/write small size objects from/to file          *
\******************************************************************/

                         /* FORWARD: default (forward scan)
                            reposition of cache as 100% forward
                            from current file position */
#define BIO_OPT_FORWARD  0x0000
#define BIO_OPT_DB       BIO_OPT_FORWARD
                         /* RANDOM: middle scan
                            reposition of cache as 50% forward & 50%
                            backward from current file position */
#define BIO_OPT_RANDOM   0x0001
                         /* BACKWARD: backward scan
                            reposition of cache as 100% backward
                            from current file position */
#define BIO_OPT_BACKSCAN 0x0002
                         /* RANDOM FORWARD: reposition of cache as 90% forward
                            & 10% backward from current file position */
#define BIO_OPT_RFORWARD 0x0003
                         /* RANDOM BACKWARD: reposition of cache as 10% forward
                            & 90% backward from current file position */
#define BIO_OPT_RBACKSCAN 0x0004
                         /* direction mask */
#define BIO_OPT_DIRMASK  0x000F
                         /* NOCACHE:  disable cache */
#define BIO_OPT_NOCACHE  0x8000

#define BIO_SEEK_SET     SEEKF_START
#define BIO_SEEK_CUR     SEEKF_CUR
#define BIO_SEEK_END     SEEKF_END

typedef void * BGLOBAL;
extern struct tagBFILE bNull; /* pointer to it return bioOpen if fail */

/*
   WARNING!!! Carefully use parameter - buffSize.
   If you create new file with 0 bytes length, then value of buffSize
   UINT_MAX - tread unbuffered i/o, this better use values < UINT_MAX.
   UINT_MAX better use for readonly operation for samll files for load
   those into memory entire.
*/

BGLOBAL               __FASTCALL__ bioOpen(const char * fname,unsigned openmode,unsigned buffSize);
tBool                 __FASTCALL__ bioChSize(BGLOBAL bioFile,unsigned long newsize);
tBool                 __FASTCALL__ bioClose(BGLOBAL bioFile);
tBool                 __FASTCALL__ bioEOF(BGLOBAL bioHandle);
unsigned long         __FASTCALL__ bioFLength(BGLOBAL bioFile);
tBool                 __FASTCALL__ bioFlush(BGLOBAL bioFile); /* Flushed buffer onto disk. This operation does automatically when called bioSeek , bioClose functions if openmode == READWRITE || WRITE */
uint8_t               __FASTCALL__ bioReadByte(BGLOBAL bioFile);
uint16_t              __FASTCALL__ bioReadWord(BGLOBAL bioFile);
uint32_t              __FASTCALL__ bioReadDWord(BGLOBAL bioFile);
tBool                 __FASTCALL__ bioReadBuffer(BGLOBAL bioFile,void * buffer,unsigned cbBuffer);
tBool                 __FASTCALL__ bioReRead(BGLOBAL bioFile); /*  if file enable write for another process reread current portion of information from disk */
tBool                 __FASTCALL__ bioSeek(BGLOBAL bioFile,long offset,int origin);
unsigned              __FASTCALL__ bioGetOptimization(BGLOBAL bioFile);
unsigned              __FASTCALL__ bioSetOptimization(BGLOBAL bioFile,unsigned flags);
unsigned long         __FASTCALL__ bioTell(BGLOBAL bioFile);
tBool                 __FASTCALL__ bioWriteByte(BGLOBAL bioFile,uint8_t bVal);
tBool                 __FASTCALL__ bioWriteWord(BGLOBAL bioFile,uint16_t wVal);
tBool                 __FASTCALL__ bioWriteDWord(BGLOBAL bioFile,uint32_t dwVal);
tBool                 __FASTCALL__ bioWriteBuffer(BGLOBAL bioFile,const void * buffer,unsigned cbBuffer);
char *                __FASTCALL__ bioFileName(BGLOBAL bioFile);
BGLOBAL               __FASTCALL__ bioDup(BGLOBAL bioFile);
BGLOBAL               __FASTCALL__ bioDupEx(BGLOBAL bioFile,unsigned buffSize);
int                   __FASTCALL__ bioHandle(BGLOBAL bioFile);

/* Below describing functions for direct access to cache */
void *                __FASTCALL__ bioBuffer(BGLOBAL bioFile);
unsigned              __FASTCALL__ bioBuffLen(BGLOBAL  bioFile);
unsigned              __FASTCALL__ bioBuffPos(BGLOBAL bioFile);

#endif

