/****************************************************************************\
*   __OS_DEP - this header file is part of Binary vIEW project (BIEW)        *
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
*   Copyrights:           1995, 2000 Nick Kurshev                            *
*   License:              See below                                          *
*   Author and developer: Nick Kurshev                                       *
*   Requirement:          ANSI C compatible development system               *
*   Original file name:   lib/sysdep/__os_dep.h                              *
*---------------------- [ D e s c r i p t i o n ] ---------------------------*
*  This file contain all operating system depended part of BIEW project.     *
*-------------------------- [ L i c e n c e ] -------------------------------*
* The Binary vIEW (BIEW) is copyright (C) 1995 Nick Kurshev.                 *
* All rights reserved. This software is redistributable under the licence    *
* given in the file "Licence.en" ("Licence.ru" in russian translation)       *
* distributed in the BIEW archive.                                           *
\****************************************************************************/

#ifndef __OS_DEP_H
#define __OS_DEP_H

#ifndef __SYS_DEP_H
#include "_sys_dep.h"
#endif

#ifndef __BIEWLIB_H
#define __NORECURSIVE
#include "biewlib.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

/* KEYBOARD handling */

#define KBD_NONSTOP_ON_MOUSE_PRESS 0x00000001L

extern void      __FASTCALL__ __init_keyboard( void );
extern void      __FASTCALL__ __term_keyboard( void );
extern int       __FASTCALL__ __kbdGetKey( unsigned long flg );
extern int       __FASTCALL__ __kbdTestKey( unsigned long flg );
extern int       __FASTCALL__ __kbdGetShiftsKey( void );

/* MOUSE handling */

#define MS_LEFTPRESS    1
#define MS_RIGHTPRESS   2
#define MS_MIDDLEPRESS  4

extern int       __FASTCALL__ __init_mouse( void );
extern void      __FASTCALL__ __term_mouse( void );
extern tBool     __FASTCALL__ __MsGetState( void ); /* return True if visible */
extern void      __FASTCALL__ __MsSetState( tBool is_visible );
extern void      __FASTCALL__ __MsGetPos( unsigned *x, unsigned *y );
extern int       __FASTCALL__ __MsGetBtns( void );

/* VIDEO subsystem handling */

/*
#define __TVIO_MAXSCREENHEIGHT  100 - No dependencies from screen height.
                                      Today project support any height of
                                      screen.
*/
#if __WORDSIZE == 16
#define __TVIO_MAXSCREENWIDTH   132
#else
#define __TVIO_MAXSCREENWIDTH   255
#endif

#define __TVIO_FLG_DIRECT_CONSOLE_ACCESS  0x00000001L
#define __TVIO_FLG_USE_7BIT               0x00000002L

extern void      __FASTCALL__ __init_vio( unsigned long flags );
extern void      __FASTCALL__ __term_vio( void );
#define __TVIO_CUR_OFF   0
#define __TVIO_CUR_NORM  1
#define __TVIO_CUR_SOLID 2
extern int       __FASTCALL__ __vioGetCursorType( void );
extern void      __FASTCALL__ __vioSetCursorType( int );
extern void      __FASTCALL__ __vioGetCursorPos(unsigned* x,unsigned* y);
extern void      __FASTCALL__ __vioSetCursorPos(unsigned x,unsigned y);
extern void      __FASTCALL__ __vioRereadState( void );
extern void      __FASTCALL__ __vioReadBuff(unsigned x,unsigned y,void *buff,unsigned len);
extern void      __FASTCALL__ __vioWriteBuff(unsigned x,unsigned y,const void *buff,unsigned len);
extern unsigned               tvioWidth,tvioHeight,tvioNumColors;
/* terminal support */
extern void      __FASTCALL__ __vioSetTransparentColor(unsigned char value);

/* OS specific functions */
extern void      __FASTCALL__ __init_sys( void );
extern void      __FASTCALL__ __term_sys( void );
extern void      __FASTCALL__ __OsYield( void );
                                            /* requires 'progname' - short name
                                               of program without suffixes.
                                               Return fully qualified name of
                                               .ini file */
extern char *    __FASTCALL__ __get_ini_name( const char *progname );
                                            /* requires 'progname' - short name
                                               of program without suffixes.
                                               Return slash terminated path to
                                               program resource dir. */
extern char *    __FASTCALL__ __get_rc_dir( const char *progname );

/* Timer functions */
/* Condition: create one timer callback per program */

typedef void timer_callback( void );
                             /* Setup user defined timer callback with given
                                time interval. Return real call back
                                interval in milliseconds */
extern unsigned  __FASTCALL__ __OsSetTimerCallBack(unsigned ms,timer_callback *func);
                             /* Restore time callback function to original
                                state */
extern void      __FASTCALL__ __OsRestoreTimer(void);


/* OS File manipulating functions */
/*
   I had problem with standard i/o function like: open, read, write
   in different C libraries such as emx and possible some one.
   It cause to born this section.
*/
extern void      __FASTCALL__ __OsClose(int handle);
extern int       __FASTCALL__ __OsChSize(int handle, long size);
extern int       __FASTCALL__ __OsCreate(const char *name);
extern int       __FASTCALL__ __OsDupHandle(int handle);
extern int       __FASTCALL__ __OsDelete(const char *name);

/* Below describing open flags that present in any OSes */
/* File open flags */
#define FO_READONLY           0x0000
#define FO_WRITEONLY          0x0001
#define FO_READWRITE          0x0002

/* sharing open flags */
#define SO_COMPAT             0x0000
#define SO_DENYALL            0x0010
#define SO_DENYWRITE          0x0020
#define SO_DENYREAD           0x0030
#define SO_DENYNONE           0x0040
#define SO_PRIVATE            0x0080

extern int       __FASTCALL__ __OsOpen(const char *name,int mode);

extern tBool  __FASTCALL__    __IsFileExists(const char *name);
extern long   __FASTCALL__    __FileLength(int handle);

#define SEEKF_START           (int)0
#define SEEKF_CUR             (int)1
#define SEEKF_END             (int)2

extern long   __FASTCALL__    __OsSeek(int handle, long newpos, int origin);
extern long   __FASTCALL__    __OsTell(int handle);

                                        /* Different C libraries have different
                                           methods for changing size of file
                                           and in many case it undocumented */
extern int       __FASTCALL__ __OsTruncFile(int handle, unsigned long newsize);
extern int       __FASTCALL__ __OsRename(const char *oldname,const char *newname);
extern int       __FASTCALL__ __OsRead(int handle,void *buff,unsigned size);
extern int       __FASTCALL__ __OsWrite(int handle,const void *buff,unsigned size);

/* National Language Support */

extern void      __FASTCALL__ __nls_OemToOsdep(unsigned char *str,unsigned size);
extern void      __FASTCALL__ __nls_CmdlineToOem(unsigned char *str,unsigned size);
extern void      __FASTCALL__ __nls_OemToFs(unsigned char *str,unsigned size);

#ifdef __cplusplus
}
#endif

#endif



