/****************************************************************************\
*   FILEIO - this source file is part of Binary vIEW project (BIEW)          *
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
*   Copyrights:           1995, 2000 Nick Kurshev                            *
*   License:              See below                                          *
*   Author and developer: Nick Kurshev                                       *
*   Requirement:          ANSI C compatible development system               *
*   Original file name:   lib/sysdep/generic/posix/fileio.c                  *
*---------------------- [ D e s c r i p t i o n ] ---------------------------*
* implementation of file i/o routines for POSIX compatible OS.               *
*-------------------------- [ L i c e n c e ] -------------------------------*
* The Binary vIEW (BIEW) is copyright (C) 1995 Nick Kurshev.                 *
* All rights reserved. This software is redistributable under the licence    *
* given in the file "Licence.en" ("Licence.ru" in russian translation)       *
* distributed in the BIEW archive.                                           *
\****************************************************************************/

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>

#include "__os_dep.h"
#include "_sys_dep.h"

int __FASTCALL__ __OsCreate(const char *name)
{
  return open(name,O_RDWR | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
}

int __FASTCALL__ __OsDelete(const char *name)
{
  return unlink(name);
}

int __FASTCALL__ __OsRename(const char *oldname,const char *newname)
{
  return rename(oldname,newname);
}

void __FASTCALL__ __OsClose(int handle)
{
  close(handle);
}

int __FASTCALL__ __OsDupHandle(int handle)
{
  return dup(handle);
}

int __FASTCALL__ __OsOpen(const char *fname,int mode)
{
  int flags = O_RDONLY;
  struct stat st;

  if((mode & FO_WRITEONLY) == FO_WRITEONLY) flags |= O_WRONLY;
  if((mode & FO_READWRITE) == FO_READWRITE) flags |= O_RDWR;
#ifdef O_SHLOCK
  if(mode & SO_DENYWRITE) flags |= O_SHLOCK;
#endif
#ifdef O_EXLOCK
  if(mode & (SO_DENYALL | SO_DENYREAD)) flags |= O_EXLOCK;
#endif
#ifdef O_BINARY
  flags |= O_BINARY;
#endif

/* check for directory (KB) */

  if (stat(fname,&st)) return -1;
  if (S_ISDIR(st.st_mode)) { errno = EISDIR; return -1; }

  return open(fname,flags);
}

long __FASTCALL__ __OsSeek(int handle,long offset,int origin)
{
  return lseek(handle,offset,origin);
}

int __FASTCALL__ __OsTruncFile( int handle, unsigned long newsize)
{
  return ftruncate(handle,newsize);
}

int __FASTCALL__ __OsRead(int handle, void *buff, unsigned count)
{
  return read(handle,buff,count);
}

int __FASTCALL__ __OsWrite(int handle,const void *buffer, unsigned count)
{
  return write(handle,buffer,count);
}

#define BLKSIZE 32767

int __FASTCALL__ __OsChSize(int handle, long size)
{
    long length, fillsize;
    char *buf;
    int  bufsize, numtowrite;

    length=__OsSeek(handle, 0L, SEEKF_END);	   /* get current length */
    if(length == -1 || size < 0)           return -1;
    if(length >= size)			   /* truncate size */
    {
	return __OsTruncFile(handle,size);
    }
    fillsize=size-length;		    /* increase size **/
    bufsize= (int) min(fillsize, BLKSIZE);
    buf=malloc(bufsize);
    if(buf == NULL) return -1;
    memset(buf, 0, bufsize);		    /* write zeros to pad file */
    do
    {
	numtowrite= (int) min(fillsize, (long) bufsize);
	if(__OsWrite(handle, buf, numtowrite) != numtowrite)
	{
	    free(buf);
	    return -1;
	}
	fillsize-=numtowrite;

    } while(fillsize);
    free(buf);
    return 0;
}
